#include <QtGui/QGuiApplication>
#include <QQuickView>
#include <QQmlEngine>
#include <QQmlContext>
#include <QLibrary>
#include <QtQml>
#include <qpa/qplatformnativeinterface.h>

#include "qmirserver.h"
#include "application_manager.h"

int startShell(int argc, const char* argv[], ShellServerConfiguration *server);

int main(int argc, const char *argv[])
{
    QMirServer mirServer(argc, argv);
    return mirServer.runWithClient(startShell);
}

int startShell(int argc, const char* argv[], ShellServerConfiguration* server)
{
    /* Workaround Qt platform integration plugin not advertising itself
       as having the following capabilities:
        - QPlatformIntegration::ThreadedOpenGL
        - QPlatformIntegration::BufferQueueingOpenGL
    */
    setenv("QML_FORCE_THREADED_RENDERER", "1", 1);
    setenv("QML_FIXED_ANIMATION_STEP", "1", 1);

    QGuiApplication application(argc, (char**)argv);

    //resolveIconTheme();

    QStringList args = application.arguments();

    // The testability driver is only loaded by QApplication but not by QGuiApplication.
    // However, QApplication depends on QWidget which would add some unneeded overhead => Let's load the testability driver on our own.
    if (args.contains(QLatin1String("-testability"))) {
        QLibrary testLib(QLatin1String("qttestability"));
        if (testLib.load()) {
            typedef void (*TasInitialize)(void);
            TasInitialize initFunction = (TasInitialize)testLib.resolve("qt_testability_init");
            if (initFunction) {
                initFunction();
            } else {
                qCritical("Library qttestability resolve failed!");
            }
        } else {
            qCritical("Library qttestability load failed!");
        }
    }

//    bindtextdomain("unity8", translationDirectory().toUtf8().data());

    QQuickView* view = new QQuickView();
    view->setResizeMode(QQuickView::SizeRootObjectToView);
    view->setTitle("Qml Phone Shell");

    // to move into QML plugin
    ApplicationManager applicationManager(server);
    qmlRegisterType<ApplicationManager>();
    qmlRegisterType<Application>();
    qmlRegisterType<ApplicationListModel>();

    // attach server instance to the context
    view->rootContext()->setContextProperty("applicationManager", &applicationManager);

    // You will need this if you want to interact with touch-only components using a mouse
    // Needed only when manually testing on a desktop.
//    MouseTouchAdaptor *mouseTouchAdaptor = 0;
//    if (args.contains(QLatin1String("-mousetouch"))) {
//        mouseTouchAdaptor = new MouseTouchAdaptor;
//        mouseTouchAdaptor->setTargetWindow(view);
//        application.installNativeEventFilter(mouseTouchAdaptor);
//    }

    QPlatformNativeInterface* nativeInterface = QGuiApplication::platformNativeInterface();
    /* Shell is declared as a system session so that it always receives all
       input events.
       FIXME: use the enum value corresponding to SYSTEM_SESSION_TYPE (= 1)
       when it becomes available.
    */
    nativeInterface->setProperty("ubuntuSessionType", 1);
    view->setProperty("role", 2); // INDICATOR_ACTOR_ROLE

    QObject::connect(view->engine(), SIGNAL(quit()), qApp, SLOT(quit()));

    QUrl source("main.qml");
//    view->engine()->addImportPath(shellAppDirectory());
//    view->engine()->addImportPath(shellImportPath());
//    appendImportPath(view->engine(), fakePluginsImportPath());
    view->setSource(source);
    view->setColor("transparent");

    view->showFullScreen();

    return application.exec();

//    delete mouseTouchAdaptor;
}
