// This file is part of QtUbuntu, a set of Qt components for Ubuntu.
// Copyright © 2013 Canonical Ltd.
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 3, as published by
// the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
// SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef APPLICATION_H
#define APPLICATION_H

#include <memory>
#include <QtCore/QtCore>

class DesktopFileReader;
namespace mir { namespace shell { class ApplicationSession; }}

class Application : public QObject {
    Q_OBJECT
    Q_ENUMS(Stage)
    Q_ENUMS(State)
    Q_PROPERTY(QString desktopFile READ desktopFile NOTIFY desktopFileChanged)
    Q_PROPERTY(QString appId READ appId NOTIFY appIdChanged)
    Q_PROPERTY(QString name READ name NOTIFY nameChanged)
    Q_PROPERTY(QString comment READ comment NOTIFY commentChanged)
    Q_PROPERTY(QString icon READ icon NOTIFY iconChanged)
    Q_PROPERTY(QString exec READ exec NOTIFY execChanged)
    Q_PROPERTY(Stage stage READ stage NOTIFY stageChanged)
    Q_PROPERTY(State state READ state NOTIFY stateChanged)
    Q_PROPERTY(bool focus READ focus NOTIFY focusChanged)
    Q_PROPERTY(bool fullscreen READ fullscreen NOTIFY fullscreenChanged)

public:
    enum Stage { MainStage, SideStage };
    enum State { Starting, Running };

    Application(DesktopFileReader* desktopData, qint64 pid, Stage stage, State state);
    ~Application();

    QString desktopFile() const;
    QString appId() const;
    QString name() const;
    QString comment() const;
    QString icon() const;
    QString exec() const;
    Stage stage() const;
    State state() const;
    bool focus() const;
    bool fullscreen() const;
    std::shared_ptr<mir::shell::ApplicationSession> session() const;

Q_SIGNALS:
    void desktopFileChanged();
    void appIdChanged();
    void nameChanged();
    void commentChanged();
    void iconChanged();
    void execChanged();
    void handleChanged();
    void stageChanged();
    void stateChanged();
    void focusChanged();
    void fullscreenChanged();

private:
    void setStage(Stage stage);
    void setState(State state);
    void setFocus(bool focus);
    void setFullscreen(bool fullscreen);
    void setSession(const std::shared_ptr<mir::shell::ApplicationSession>& session);
    void setSessionName(const QString& name);

    DesktopFileReader* m_desktopData;
    qint64 m_pid;
    Stage m_stage;
    State m_state;
    bool m_focus;
    bool m_fullscreen;
    std::shared_ptr<mir::shell::ApplicationSession> m_session;
    QString m_sessionName;

    friend class ApplicationManager;
    friend class ApplicationListModel;
};

Q_DECLARE_METATYPE(Application*)

#endif  // APPLICATION_H
