#include "mirsurface.h"

MirSurface::MirSurface(std::weak_ptr<mir::shell::Surface> surface, QQuickItem *parent) :
    QQuickItem(parent)
  , m_surface(surface)
{
    setFlag(QQuickItem::ItemHasContents, false); //so scene graph will not render this item
}

qreal MirSurface::x() const {
    if (auto surface = m_surface.lock()) {
        return surface->top_left().x.as_int();
    } else {
        return 0;
    }
}

qreal MirSurface::y() const {
    if (auto surface = m_surface.lock()) {
        return surface->top_left().y.as_int();
    } else {
        return 0;
    }
}

void MirSurface::setX(qreal xValue) {
    using namespace mir::geometry;

    if (auto surface = m_surface.lock()) {
        Point position = surface->top_left();
        if (position.x.as_int() != (int) xValue) {
            surface->move_to(Point{ X{xValue}, position.y });
            Q_EMIT yChanged();
        }
    }
}

void MirSurface::setY(qreal yValue) {
    using namespace mir::geometry;

    if (auto surface = m_surface.lock()) {
        Point position = surface->top_left();
        if (position.y.as_int() != (int) yValue) {
            surface->move_to(Point{ position.x, Y{yValue} });
            Q_EMIT yChanged();
        }
    }
}

qreal MirSurface::width() const {
    if (auto surface = m_surface.lock()) {
        return surface->size().width.as_int();
    } else {
        return 0;
    }
}

qreal MirSurface::height() const {
    if (auto surface = m_surface.lock()) {
        return surface->size().height.as_int();
    } else {
        return 0;
    }
}

void MirSurface::setWidth(qreal widthValue) {
    using namespace mir::geometry;

    if (auto surface = m_surface.lock()) {
        Size size = surface->size();
        if (size.width.as_int() != (int) widthValue) {
//            m_surface->set_size(Size(widthValue, size.height))); //MISSING FROM API
            Q_EMIT widthChanged();
        }
    }
}

void MirSurface::setHeight(qreal heightValue) {
    using namespace mir::geometry;

    if (auto surface = m_surface.lock()) {
        Size size = surface->size();
        if (size.height.as_int() != (int) heightValue) {
//            m_surface->set_size(Size(size.width, heightValue))); //MISSING FROM API
            Q_EMIT heightChanged();
        }
    }
}


bool MirSurface::isVisible() const {
    if (auto surface = m_surface.lock()) {
        return surface->visible();
    } else {
        return false;
    }
}

void MirSurface::setVisible(const bool visible) {
    if (auto surface = m_surface.lock()) {
        if (visible == surface->visible()) return;

        if (visible) {
            surface->show();
        } else {
            surface->hide();
        }
        Q_EMIT visibleChanged();
    }
}


MirSurface::Type MirSurface::type() const { //FIXME - how to listen to change in this property
    if (auto surface = m_surface.lock()) {
        return static_cast<MirSurface::Type>(surface->type());
    } else {
        return Normal;
    }
}

MirSurface::State MirSurface::state() const { //FIXME - how to listen to change in this property
    if (auto surface = m_surface.lock()) {
        return static_cast<MirSurface::State>(surface->state());
    } else {
        return Unknown;
    }
}

QString MirSurface::name() const { //FIXME - how to listen to change in this property
    if (auto surface = m_surface.lock()) {
        return QString::fromStdString(surface->name());
    } else {
        return QString();
    }
}
