#ifndef MIRSURFACE_H
#define MIRSURFACE_H

#include <QQuickItem>
#include <mirserver/mir/shell/surface.h>

class MirSurface : public QQuickItem
{
    Q_OBJECT
    Q_ENUMS(Type)
    Q_ENUMS(State)

    Q_PROPERTY(Type type READ type NOTIFY typeChanged)
    Q_PROPERTY(State state READ state NOTIFY stateChanged)
    Q_PROPERTY(QString name READ name NOTIFY nameChanged)

public:
    explicit MirSurface(std::weak_ptr<mir::shell::Surface> surface, QQuickItem *parent = 0);

    enum Type {
        Normal = mir_surface_type_normal,
        Utility = mir_surface_type_utility,
        Dialog = mir_surface_type_dialog,
        Overlay = mir_surface_type_overlay,
        Freestyle = mir_surface_type_freestyle,
        Popover = mir_surface_type_popover
        };

    enum State {
        Unknown = mir_surface_state_unknown,
        Restored = mir_surface_state_restored,
        Minimized = mir_surface_state_minimized,
        Maxinized = mir_surface_state_maximized,
        VertMaximized = mir_surface_state_vertmaximized,
        /* SemiMaximized = mir_surface_state_semimaximized // see mircommon/mir_toolbox/common.h*/
        Fullscreen = mir_surface_state_fullscreen,
    };

    //getters
    Type type() const;
    State state() const;
    QString name() const;

    qreal x() const;
    qreal y() const;
    qreal width() const;
    qreal height() const;
    bool isVisible() const;

    //setters
    void setX(qreal);
    void setY(qreal);
    void setWidth(qreal);
    void setHeight(qreal);
    void setVisible(bool);

Q_SIGNALS:
    void typeChanged();
    void stateChanged();
    void nameChanged();

protected:
    std::weak_ptr<mir::shell::Surface> m_surface;
};

#endif // MIRSURFACE_H
