// Mir support
#include <mirserver/mir/run_mir.h>
#include <mirserver/mir/abnormal_exit.h>
#include <mirserver/mir/default_server_configuration.h>

// Platform API
#include "ubuntu/application/ubuntu_application_api_mirserver_priv.h"

// Qt
#include <QDebug>
#include <QThread>

// Std
#include <thread>

// local
#include "qmirserver.h"

QMirServer::QMirServer(int argc, const char* argv[], QObject *parent)
    : QObject(parent)
    , m_argc(argc)
    , m_argv(argv)
{
}

QMirServer::~QMirServer()
{
}

int QMirServer::runWithClient(std::function<int(int, const char**, ShellServerConfiguration*)> client)
{
    // need local copies as lambda function will not capture class members
    int argc = m_argc;
    auto argv = m_argv;
    auto config = new ShellServerConfiguration(m_argc, m_argv);

    mir::run_mir(*config, [config, &client, &argc, &argv](mir::DisplayServer&) {
        ua_ui_mirserver_init(*config);

        try {
            new std::thread(client, argc, argv, config);
        } catch (...) {
            qDebug() << "Exception caught, quitting";
        }
    });
    ua_ui_mirserver_finish();
    delete config;
    return 0;
}
