#include "sessionlistener.h"
#include "logging.h"

#include <mir/shell/application_session.h>

namespace msh = mir::shell;

Q_DECLARE_METATYPE(std::shared_ptr<msh::ApplicationSession>)

SessionListener::SessionListener(QObject *parent) :
    QObject(parent)
{
    DLOG("SessionListener::SessionListener (this=%p)", this);
    // need to register type to send over threads with signal/slot
    qRegisterMetaType<std::shared_ptr<msh::ApplicationSession>>("std::shared_ptr<mir::shell::ApplicationSession>");
}

SessionListener::~SessionListener()
{
    DLOG("SessionListener::~SessionListener (this=%p)", this);
}

// Mir's default SurfaceManager calls these methods with instance of msh::ApplicationSession
// it is not a plain msh::Session
void SessionListener::starting(std::shared_ptr<msh::Session> const& session)
{
    DLOG("SessionListener::starting (this=%p, session=%p)", this, (void*)session.get());
    Q_EMIT sessionStarting(std::static_pointer_cast<msh::ApplicationSession>(session));
}

void SessionListener::stopping(std::shared_ptr<msh::Session> const& session)
{
    DLOG("SessionListener::stopping (this=%p, session=%p)", this, (void*)session.get());
    Q_EMIT sessionStopping(std::static_pointer_cast<msh::ApplicationSession>(session));
}

void SessionListener::focused(std::shared_ptr<msh::Session> const& session)
{
    DLOG("SessionListener::focused (this=%p, session=%p)", this, (void*)session.get());
    Q_EMIT sessionFocused(std::static_pointer_cast<msh::ApplicationSession>(session));
}

void SessionListener::unfocused()
{
    DLOG("SessionListener::unfocused (this=%p)", this);
    Q_EMIT sessionUnfocused();
}
