#include "shellserverconfiguration.h"

#include "initialsurfaceplacementstrategy.h"
#include "sessionlistener.h"
#include "surfacesource.h"
// #include "sessionauthorizer.h"

// mir
#include "mir/shell/organising_surface_factory.h"

// Qt
#include <QDebug>

namespace msh = mir::shell;

ShellServerConfiguration::ShellServerConfiguration(int argc, char const* argv[], QObject* parent)
    : QObject(parent)
    , DefaultServerConfiguration(argc, argv)
    , m_sessionListener(new SessionListener)
    , m_surfaceSource(new SurfaceSource(the_surface_builder()))
{
    qDebug() << "ShellServerConfiguration created";
}

ShellServerConfiguration::~ShellServerConfiguration()
{
}

std::shared_ptr<msh::PlacementStrategy>
ShellServerConfiguration::the_shell_placement_strategy()
{
    return shell_placement_strategy(
        [this]
        {
            return std::make_shared<InitialSurfacePlacementStrategy>(the_display());
        });
}

std::shared_ptr<msh::SessionListener>
ShellServerConfiguration::the_shell_session_listener()
{
    std::shared_ptr<SessionListener> sessionListener = m_sessionListener;
    return shell_session_listener(
        [this, &sessionListener]
        {
            return sessionListener;
        });
}

std::shared_ptr<msh::SurfaceFactory>
ShellServerConfiguration::the_shell_surface_factory()
{
    return shell_surface_factory(
        [this]
        {
            return std::make_shared<mir::shell::OrganisingSurfaceFactory>(
                m_surfaceSource,
                the_shell_placement_strategy());
        });
}

//    std::shared_ptr<mir::frontend::SessionAuthorizer> ShellServerConfiguration::the_session_authorizer()
//    {
//        return session_authorizer(
//            [this]
//            {
//                return std::make_shared<SessionAuthorizer>(the_session_manager());
//            }
//        );
//    }


SessionListener* ShellServerConfiguration::sessionListener() const
{
    return m_sessionListener.get();
}

SurfaceSource *ShellServerConfiguration::surfaceSource() const
{
    return m_surfaceSource.get();
}
